"""Tests for the form mixins of the ``multilingual_tags`` app."""
from django.test import TestCase

from ..models import Tag, TaggedItem
from .factories import DummyModelFactory
from .test_app.forms import DummyModelForm, LimitedDummyModelForm
from .test_app.models import DummyModel


class TaggingFormMixinTestCase(TestCase):
    """Tests for the ``TaggingFormMixin`` form mixin."""
    longMessage = True

    def setUp(self):
        self.dummy = DummyModelFactory()
        self.data = {
            'charfield': u'foobar',
            'tags': u'tagging, test'
        }

    def test_mixin(self):
        form = DummyModelForm(data=self.data, instance=self.dummy)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))

        form.save()
        self.assertEqual(DummyModel.objects.count(), 1, msg=(
            'The form should have saved one dummy.'))
        self.assertEqual(TaggedItem.objects.count(), 2, msg=(
            'The form should create one TaggedItem per entered tag.'))
        self.assertEqual(Tag.objects.count(), 2, msg=(
            'The form should create one Tag per entered tag.'))

        self.assertEqual(
            TaggedItem.objects.all()[0].user, self.dummy.user,
            msg='The form should have added the user to the taggeditems.')

        form = LimitedDummyModelForm(
            data=self.data, instance=self.dummy)
        self.assertFalse(form.is_valid(), msg=(
            'The form should not be valid when there are too many tags.'
        ))



"""Just dummy forms to test."""
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import DummyModel
from ...forms.mixins import TaggingFormMixin


class DummyModelForm(TaggingFormMixin, forms.ModelForm):

    class Meta:
        model = DummyModel
        fields = ('charfield', )


class LimitedDummyModelForm(TaggingFormMixin, forms.ModelForm):

    tag_field = {
        'name': 'tags',
        'label': _('Tags'),
        'help_text': _('Add tags separated by comma.'),
        'required': True,
        'max_tags': 1,
    }

    class Meta:
        model = DummyModel
        fields = ('charfield', )



"""Factories for the multilingual_tags app."""
from django.conf import settings

from django_libs.tests.factories import HvadFactoryMixin, UserFactory

import factory

from .. import models
from .test_app.models import DummyModel


class DummyModelFactory(factory.DjangoModelFactory):
    """Factory for the ``DummyModel`` test model."""
    FACTORY_FOR = DummyModel

    charfield = factory.Sequence(lambda n: 'charfield {0}'.format(n))
    user = factory.SubFactory(UserFactory)


class TagFactory(HvadFactoryMixin, factory.DjangoModelFactory):
    """Factory for the ``Tag`` model."""
    FACTORY_FOR = models.Tag

    slug = factory.Sequence(lambda n: 'slug-{0}'.format(n))
    name = factory.Sequence(lambda n: 'name {0}'.format(n))
    language_code = settings.LANGUAGE_CODE


class TaggedItemFactory(factory.DjangoModelFactory):
    """Factory for the ``TaggedItem`` model."""
    FACTORY_FOR = models.TaggedItem

    tag = factory.SubFactory(TagFactory)
    object = factory.SubFactory(DummyModelFactory)



"""Admin classes for the multilingual_tags app."""
from django import forms
from django.contrib import admin
from django.contrib.contenttypes.generic import (
    BaseGenericInlineFormSet,
    GenericTabularInline,
)
from django.utils.translation import ugettext_lazy as _

from hvad.admin import TranslatableAdmin

from . import models


class TaggedItemInlineFormSet(BaseGenericInlineFormSet):
    model = models.TaggedItem

    def clean(self):
        cleaned_data = super(TaggedItemInlineFormSet, self).clean()
        # validate, that every tag is only used once
        tags = []
        for i in range(0, self.total_form_count()):
            tags.append(self.data.get('{0}-{1}-tag'.format(self.prefix, i)))
        if len(tags) != len(set(tags)):
            raise forms.ValidationError(_(
                'A Tag may only exist once per object.'))
        return cleaned_data


class TaggedItemInline(GenericTabularInline):
    formset = TaggedItemInlineFormSet
    model = models.TaggedItem
    extra = 1


admin.site.register(models.Tag, TranslatableAdmin)
admin.site.register(models.TaggedItem)



"""Just a dummy."""
from django.db import models
from django.conf import settings
from django.contrib.contenttypes import generic


class DummyModel(models.Model):
    charfield = models.CharField(max_length=64)
    tags = generic.GenericRelation('multilingual_tags.TaggedItem')
    user = models.ForeignKey(settings.AUTH_USER_MODEL)

    def get_user(self):
        return self.user



"""Test admins"""
from django.contrib import admin

from multilingual_tags.admin import TaggedItemInline

from . import models


class DummyAdmin(admin.ModelAdmin):
    inlines = [TaggedItemInline]

admin.site.register(models.DummyModel, DummyAdmin)



"""Form mixins for the ``multilingual_tags`` app."""
from django import forms
from django.forms.fields import ErrorList
from django.contrib.contenttypes.models import ContentType
from django.utils.text import slugify
from django.utils.translation import get_language, ugettext_lazy as _

from .. import models


class TaggingFormMixin(object):
    """Mixin for ModelForms to add multilingual tags to a model."""
    tag_field = {
        'name': 'tags',
        'label': _('Tags'),
        'help_text': _('Add tags separated by comma.'),
        'required': True,
        'max_tags': 0,
    }

    def __init__(self, *args, **kwargs):
        super(TaggingFormMixin, self).__init__(*args, **kwargs)
        self._taggeditems = []
        self._instance_ctype = None
        self.fields[self._get_tag_field_name()] = forms.CharField(
            label=self._get_tag_field_label(),
            help_text=self._get_tag_field_help_text(),
            initial=self._get_tag_field_initial(),
            required=self._get_tag_field_required(),
        )
        self.fields[self._get_tag_field_name()].widget.attrs.update({
            'data-class': 'multilingual-tags-field',
            'data-max-tags': self._get_tag_field_max_tags()})
        setattr(self, 'clean_{0}'.format(self._get_tag_field_name()),
                self._get_tag_field_clean())

    def add_error(self, fieldname, message):
        if fieldname in self._errors:
            self._errors[fieldname].append(message)
        else:
            self._errors[fieldname] = ErrorList()
            self._errors[fieldname].append(message)

    def _get_tag_field_clean(self):
        def clean_field():
            self._tags_added = []
            self._taggeditems = []
            language = get_language()
            max_tags = self._get_tag_field_max_tags()

            data = self.data.get(self._get_tag_field_name())
            if not data:
                return []
            tag_data = [t.strip() for t in data.split(',')]
            self._instance_ctype = ContentType.objects.get_for_model(
                self.instance)
            for tag_string in tag_data:
                if len(tag_string) > 64:
                    self.add_error(
                        self._get_tag_field_name(),
                        _('Tags cannot be longer than 64 characters:'
                          ' "{0}"'.format(tag_string))
                    )
                    continue
                try:
                    tag = models.Tag.objects.get(
                        slug=slugify(tag_string))
                except models.Tag.DoesNotExist:
                    # TODO tags should not be stored directly
                    tag = models.Tag.objects.create(
                        slug=slugify(tag_string),
                        name=tag_string,
                        language_code=language)
                # prevent duplicate tags
                if tag not in self._tags_added:
                    self._tags_added.append(tag)
                    if self.instance.id:
                        taggeditem, created = (
                            models.TaggedItem.objects.get_or_create(
                                tag=tag,
                                content_type=self._instance_ctype,
                                object_id=self.instance.id,
                            )
                        )
                    else:
                        taggeditem = models.TaggedItem(
                            tag=tag,
                            content_type=self._instance_ctype)
                    self._taggeditems.append(taggeditem)
                if max_tags and len(self._tags_added) > max_tags:
                    self.add_error(
                        self._get_tag_field_name(),
                        _('You cannot add more than {0} tags.'.format(
                            self._get_tag_field_max_tags()
                        ))
                    )
            return self._taggeditems
        return clean_field

    def _get_tag_field_help_text(self):
        return self.tag_field.get('help_text', '')

    def _get_tag_field_initial(self):
        tag_model_field = getattr(self.instance, self._get_tag_field_name())
        return ','.join([ti.tag.name for ti in tag_model_field.all()])

    def _get_tag_field_label(self):
        return self.tag_field.get('label', 'Tags')

    def _get_tag_field_max_tags(self):
        return int(self.tag_field.get('max_tags', 0))

    def _get_tag_field_name(self):
        return self.tag_field.get('name', 'tags')

    def _get_tag_field_required(self):
        return self.tag_field.get('required', True)

    def save(self, commit=True):
        instance = super(TaggingFormMixin, self).save(commit)
        for item in self._taggeditems:
            if hasattr(instance, 'get_user'):
                item.user = instance.get_user()
            item.object_id = instance.id
            item.save()
        models.TaggedItem.objects.filter(
            content_type=self._instance_ctype,
            object_id=instance.id).exclude(
            pk__in=[ti.pk for ti in self._taggeditems]).delete()
        return instance



#!/usr/bin/env python
"""
This script is a trick to setup a fake Django environment, since this reusable
app will be developed and tested outside any specifiv Django project.

Via ``settings.configure`` you will be able to set all necessary settings
for your app and run the tests as if you were calling ``./manage.py test``.

"""
import sys

from django.conf import settings

import coverage

import test_settings


if not settings.configured:
    settings.configure(**test_settings.__dict__)


from django_coverage.coverage_runner import CoverageRunner
from django_nose import NoseTestSuiteRunner


class NoseCoverageTestRunner(CoverageRunner, NoseTestSuiteRunner):
    """Custom test runner that uses nose and coverage"""
    def run_tests(self, *args, **kwargs):
        results = super(NoseCoverageTestRunner, self).run_tests(
            *args, **kwargs)
        coverage._the_coverage.data.write_file('.coverage')
        return results


def runtests(*test_args):
    failures = NoseCoverageTestRunner(verbosity=2, interactive=True).run_tests(
        test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



"""Tests for the models of the multilingual_tags app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from .test_app.models import DummyModel
from django_libs.tests.factories import UserFactory

from . import factories
from .. import models


class DummyModelTestCase(TestCase):
    """Tests for the ``DummyModel`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instantiation of the ``DummyModel`` model."""
        dummymodel = factories.DummyModelFactory()
        self.assertTrue(dummymodel.pk)


class TagTestCase(TestCase):
    """Tests for the ``Tag`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instantiation of the ``Tag`` model."""
        tag = factories.TagFactory()
        self.assertTrue(tag.pk)


class TaggedItemTestCase(TestCase):
    """Tests for the ``TaggedItem`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instantiation of the ``TaggedItem`` model."""
        taggeditem = factories.TaggedItemFactory()
        self.assertTrue(taggeditem.pk)


class TagManagerTestCase(TestCase):
    """Tests for the `TagManager` manager class."""
    longMessage = True

    def setUp(self):
        self.tagged_item = factories.TaggedItemFactory()
        self.dummy = self.tagged_item.object
        self.tag = self.tagged_item.tag
        self.other_tagged_item = factories.TaggedItemFactory(object=self.dummy)

        factories.TaggedItemFactory()

        self.user_item = factories.TaggedItemFactory(object=UserFactory())

    def test_manager(self):
        self.assertEqual(
            list(models.Tag.objects.get_for_obj(self.dummy)),
            list(models.Tag.objects.filter(
                tagged_items__object_id=self.dummy.id,
                tagged_items__content_type=ContentType.objects.get_for_model(
                    DummyModel))),
            msg='Expected different tags from the manager.')

        self.assertEqual(
            list(models.Tag.objects.get_for_queryset(
                DummyModel.objects.all())),
            list(models.Tag.objects.exclude(
                tagged_items__id=self.user_item.id)),
            msg='Expected different tags from the manager.')



"""Models for the `multilingual_tags` app."""
from django.contrib.contenttypes import generic, models as ctype_models
from django.db import models
from django.utils.translation import ugettext_lazy as _, get_language
from django.conf import settings

from hvad.models import TranslatableModel, TranslatedFields, TranslationManager


class TagManager(TranslationManager):
    """Manager for the `Tag` model."""
    def get_for_model(self, obj):
        """Returns the tags for a specific model/content type."""
        qs = Tag.objects.language(get_language())
        qs = qs.filter(
            tagged_items__content_type=ctype_models.ContentType.objects.get_for_model(obj))  # NOQA
        return qs

    def get_for_obj(self, obj):
        """Returns the tags for a specific object."""
        qs = Tag.objects.language(get_language())
        qs = qs.filter(
            tagged_items__object_id=obj.id,
            tagged_items__content_type=ctype_models.ContentType.objects.get_for_model(obj))  # NOQA
        return qs

    def get_for_queryset(self, obj_queryset):
        """Returns all tags for a whole queryset of objects."""
        qs = Tag.objects.language(get_language())
        if obj_queryset.count() == 0:
            return qs.none()
        qs = qs.filter(
            tagged_items__object_id__in=[
                obj.id for obj in obj_queryset],
            tagged_items__content_type=ctype_models.ContentType.objects.get_for_model(obj_queryset[0]))  # NOQA
        return qs.distinct()


class Tag(TranslatableModel):
    """
    The information about the tag itself.

    :slug: A unique slug.

    translated:
    :name: A translatable name of the tag.

    """

    slug = models.SlugField(
        verbose_name=_('Slug'),
        max_length=64,
        unique=True,
    )

    translations = TranslatedFields(
        name=models.CharField(
            verbose_name=_('Name'),
            max_length=64,
        ),
    )

    objects = TagManager()

    def __unicode__(self):
        return self.safe_translation_getter('name', self.slug)


class TaggedItem(models.Model):
    """
    Intermediary model to attach a `Tag` to any other model instance.

    :tag: FK to the `Tag` that is atttached.
    :object: GFK to the other model instance, which is being tagged.

    """

    tag = models.ForeignKey(
        Tag,
        verbose_name=_('Tag'),
        related_name='tagged_items',
    )

    content_type = models.ForeignKey(
        ctype_models.ContentType,
        related_name='tagged_items',
    )
    object_id = models.PositiveIntegerField()
    object = generic.GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_('user'),
        related_name='tagged_items',
        blank=True, null=True,
    )

    def __unicode__(self):
        return u'{0}: #{1}'.format(self.object, self.tag)

    class Meta:
        unique_together = ('content_type', 'object_id', 'tag')



# flake8: noqa
# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

try:
    from django.contrib.auth import get_user_model
except ImportError:  # Django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()


USER_MODEL = {
    'orm_label': '%s.%s' % (User._meta.app_label, User._meta.object_name),
    'model_label': '%s.%s' % (User._meta.app_label, User._meta.module_name),
    'object_name': User.__name__,
}


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'TagTranslation'
        db.create_table(u'multilingual_tags_tag_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=64)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['multilingual_tags.Tag'])),
        ))
        db.send_create_signal(u'multilingual_tags', ['TagTranslation'])

        # Adding unique constraint on 'TagTranslation', fields ['language_code', 'master']
        db.create_unique(u'multilingual_tags_tag_translation', ['language_code', 'master_id'])

        # Adding model 'Tag'
        db.create_table(u'multilingual_tags_tag', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=64)),
        ))
        db.send_create_signal(u'multilingual_tags', ['Tag'])

        # Adding model 'TaggedItem'
        db.create_table(u'multilingual_tags_taggeditem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('tag', self.gf('django.db.models.fields.related.ForeignKey')(related_name='tagged_items', to=orm['multilingual_tags.Tag'])),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'multilingual_tags', ['TaggedItem'])

        # Adding unique constraint on 'TaggedItem', fields ['content_type', 'object_id', 'tag']
        db.create_unique(u'multilingual_tags_taggeditem', ['content_type_id', 'object_id', 'tag_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'TaggedItem', fields ['content_type', 'object_id', 'tag']
        db.delete_unique(u'multilingual_tags_taggeditem', ['content_type_id', 'object_id', 'tag_id'])

        # Removing unique constraint on 'TagTranslation', fields ['language_code', 'master']
        db.delete_unique(u'multilingual_tags_tag_translation', ['language_code', 'master_id'])

        # Deleting model 'TagTranslation'
        db.delete_table(u'multilingual_tags_tag_translation')

        # Deleting model 'Tag'
        db.delete_table(u'multilingual_tags_tag')

        # Deleting model 'TaggedItem'
        db.delete_table(u'multilingual_tags_taggeditem')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'multilingual_tags.tag': {
            'Meta': {'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '64'})
        },
        u'multilingual_tags.taggeditem': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'tag'),)", 'object_name': 'TaggedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'tagged_items'", 'to': u"orm['multilingual_tags.Tag']"})
        },
        u'multilingual_tags.tagtranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'TagTranslation', 'db_table': "u'multilingual_tags_tag_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['multilingual_tags.Tag']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '64'})
        }
    }

    complete_apps = ['multilingual_tags']
